-- $Id: BaffleFuncs_02.lua,v 1.2 2007/07/30 23:43:50 dj Exp $

$ifnot BaffleFuncs

BaffleFuncs = {}


--------------------------------------------------------------------------
-- BaffleFuncs.construct_displacement
-- 
-- construct displacement
--
-- Inputs:
--
--   delta_X     -- X_AXAF; optical axis, +ve in direction from focal plane
--                          towards source
--   delta_Y     -- Y_AXAF; grating dispersion direction
--   delta_Z     -- Z_AXAF; SIM translation axis; +ve in anti-solar direction
--
-- returns:
--   displacement   -- table { x, y, z } in raytrace coordinates
--                     z is optical axis; +ve from source towards focal plane

function BaffleFuncs.construct_displacement( delta_X, delta_Y, delta_Z, 
                                             coord_type )

  local displacement = {}
  displacement.dx =  delta_Y
  displacement.dy = -delta_Z
  displacement.dz = -delta_X

  -- print( "dsp orig ", delta_X, delta_Y, delta_Z )
  -- print( "dsp xfrm ", displacement.dx, displacement.dy, displacement.dz )

  return displacement

end

--------------------------------------------------------------------------
-- BaffleFuncs.construct_disorientation
-- 
-- construct disorientation
--
-- Inputs:
--
--   azmis_H        -- (arcmin) rotation by azmis_H about the -Z_H axis, 
--                              taking Y_H to YP and X_H to XP; the 
--                              ZP axis is the same as the Z axis.
--                              Angle is measured relative to +Y_H.
--                              Positive azmis_H rotates +Y_H towards +X_H.
--                              This is the same sense of rotation 
--                              as HRMA yaw.
--   elmis_H        -- (arcmin) Rotation by elmis_H about the -YP axis 
--                              (taking XP to XPP and ZP (= Z_H) to ZPP; 
--                              the YPP axis is the same as the YP axis.
--                              Angle is measured relative to +Z_H.
--                              Positive azmis_H rotates +ZP towards -XP.
--                              This is the same sense as HRMA pitch.
--   clock_H        -- (deg)    Rotate by clock_H about the -XPP axis;
--                              Angle is measured relative to +Y_H.
--                              Positive clock_H rotates +YPP towards -ZPP.
--
-- returns:
--   disorientation -- table { azmis, elmis, clock } in raytrace coordinates
--                     HRMA coordinates

function BaffleFuncs.construct_disorientation( azmis_H, elmis_H, clock_H, 
                                              coord_type )

  local disorientation = {}
  disorientation.azmis =  BaffleFuncs.arcmin2rad() * azmis_H
  disorientation.elmis =  BaffleFuncs.arcmin2rad() * elmis_H
  disorientation.clock =  BaffleFuncs.deg2rad()    * clock_H

  -- print( "ang orig ", azmis_H, elmis_H, clock_H )
  -- print( "ang xfrm ", disorientation.azmis, disorientation.elmis, disorientation.clock )

  return disorientation

end

--------------------------------------------------------------------------
-- BaffleFuncs.apply_displacement_disorientation
--
-- move a baffle according to the specified displacement/disorientation.
--
-- NOTE:  these are in raytrace coordinates.
--
-- Inputs:
--   displacement
--   disorientation
--
-- where
--
--   displacement {
--     dx              -- (mm)           decenter
--     dy              -- (mm)           decenter
--     dz              -- (mm)           despace
--   }
--   disorientation {
--     azmis           -- (radians)
--     elmis           -- (radians)
--     clock           -- (radians)
--   }

function BaffleFuncs.apply_displacement_disorientation( 
            displacement, disorientation 
)

  -- Motion should happen in the Body Centered Coordinate System (OSAC v7 p4-4, BCS)

  -- NOTE: angles need to be in radians for standard 'Aperture' package routines.
  -- NOTE: distances are in mm.

  -- Variable 'motion' : function   name                performing
  -- ================    --------   ----                ----------
  --                     decenter   dx, dy            X, Y translations
  --                     despace    dz                Z translation
  --                     tilt       azmis, elmis      azmis, elmis rotations (radians)
  --                     clock      clock             clock rotation around Z (radians)

  -- Still TB checked : for the right order: OSAC requires :
  -- 1) Rotate around old OY with azmis
  -- 2) Rotate around new OX with elmis
  -- 3) Rotate around new OZ with clock  (TG)

  rotate_y( disorientation.azmis )

  --  This rotation should be around 'old' OX axis

  -- push_ctm()    -- (push current transformation matrix)
  rotate_x( disorientation.elmis )
  -- pop_ctm()     -- (pop current transformation matrix)

  --  This rotation should be around 'new' OZ axis      (clock)

  rotate_z( disorientation.clock )

  --  This translation should be along 'old' OX OY axes (decenter)
  --  and along 'old' OZ axis (despace)

  -- push_ctm()    -- (push current transformation matrix)
  translate( displacement.dx, displacement.dy, displacement.dz )
  -- pop_ctm()     -- (pop current transformation matrix)

end  -- end of function 'BaffleFuncs.move()'


--------------------------------------------------------------------------
-- BaffleFuncs.construct_annulus
--
--
-- Inputs:
--
--   data -- table with info read from baffle configuration file

function BaffleFuncs.construct_annulus( data )
       
   local types = { none = -1, transp = 0, opaque = 1 }

   local type = types[data.annulus_type]

   if nil == type then
      error ( 'BaffleSet:construct_annulus: invalid annulus type: ' .. data.annulus_type )
   end

   local annulus_info = {}

   if ( 'none' == data.annulus_type ) then
      annulus_info = {  ri = nil, ro = nil, dri = nil, dro = nil, type = type }
   else
      annulus_info = {  ri   = data.R_i,      ro  = data.R_o,
	                dri  = data.tol_R_i,  dro = data.tol_R_o, 
                        type = type }
   end

   return annulus_info

end

--------------------------------------------------------------------------
-- BaffleFuncs.construct_struts
--
--
-- Inputs:
--
--   data -- table with info read from baffle configuration file

function BaffleFuncs.construct_struts( data )

   local strut_info = {}
   
   if ( 'none'  == data.strut_type ) then
   
      strut_info = {
	 num         = 0,
	 width       = nil,
	 delta_theta = nil,
	 theta0      = nil,
	 R_i         = nil,
	 R_o         = nil,
	 type        = data.strut_type 
      }
   
   elseif ( 'strut' == data.strut_type ) then
   
      strut_info = {
	 num         = data.strut_num,
	 width       = data.strut_width,
	 delta_theta = BaffleFuncs.deg2rad() * data.strut_dtheta,
	 theta0      = BaffleFuncs.deg2rad() * data.strut_theta0,
	 R_i         = nil,
	 R_o         = nil,
	 type        = data.strut_type 
      }

   elseif ( 'rect' == data.strut_type ) then

      strut_info = {
	 num         = data.strut_num,
	 width       = data.strut_width,
	 delta_theta = BaffleFuncs.deg2rad() * data.strut_dtheta,
	 theta0      = BaffleFuncs.deg2rad() * data.strut_theta0,
	 R_i         = data.strut_R_i,
	 R_o         = data.strut_R_o,
	 type        = data.strut_type 
      }

   else
      
      error ( 'BaffleSet:construct_struts: invalid strut type: ' .. data.strut_type )
     
   end

   return strut_info

end

--------------------------------------------------------------------------
-- BaffleFuncs.do_strutpairs
--
-- do a pair of struts
--
-- Inputs:
--
-- strut_info {
--     num            --    number of struts
--     width          -- (mm)       strut width
--     R_i            -- (mm)       innermost radius for strut
--     R_o            -- (mm)       outermost radius for strut
--     delta_theta    -- (radians)  angle between strut centerlines
--     theta0         -- (radians)  offset of first strut
--     type           -- strut | rect
-- }

function BaffleFuncs.do_strutpairs( strut_info )

  if ( nil ~= strut_info ) then
    if ( strut_info.num > 0 ) then
      begin_subassembly( )
        rotate_z ( strut_info.theta0 )
        ns = strut_info.num / 2
        n  = 0
        while n < ns do
          strut( strut_info.width, 1, 0 )
          rotate_z ( strut_info.delta_theta )
          n = n + 1
        end
      end_subassembly( )
    end
  end

end

--------------------------------------------------------------------------
-- BaffleFuncs.do_strut
--
-- do a single strut
--
-- Inputs:
--
-- strut_info {
--     num            --    number of struts
--     width          -- (mm)       strut width
--     R_i            -- (mm)       innermost radius for strut
--     R_o            -- (mm)       outermost radius for strut
--     delta_theta    -- (radians)  angle between strut centerlines
--     theta0         -- (radians)  offset of first strut
--     type           -- strut | rect
-- }

function BaffleFuncs.do_strut( strut_info )

  if ( (nil ~= strut_info) and (strut_info.num > 0) ) then
    begin_subassembly( )
      rotate_z ( strut_info.theta0 )
      ns = strut_info.num / 2
      n  = 0
      while n < ns do
        --  print_xfrm( )              -- for tests only
        strut( strut_info.width, 1, 0 )
        rotate_z ( strut_info.delta_theta )
        -- print_xfrm( )              -- for tests only
        n = n + 1
      end
    end_subassembly( )
  end

end

------------------------------------------------------------------------------
-- BaffleFuncs.do_rect_strut
--
-- do a rectangular strut
--
-- Inputs:
--
-- strut_info {
--     num            --    number of struts
--     width          -- (mm)       strut width
--     R_i            -- (mm)       innermost radius for strut
--     R_o            -- (mm)       outermost radius for strut
--     delta_theta    -- (radians)  angle between strut centerlines
--     theta0         -- (radians)  offset of first strut
--     type           -- strut | rect
-- }

function BaffleFuncs.do_rect_strut( strut_info )

  if ( strut_info.num > 0 ) then

    local xcen = 0.5 * (strut_info.R_o + strut_info.R_i)
    local slen =        strut_info.R_o - strut_info.R_i

    begin_subassembly( )
      rotate_z ( strut_info.theta0 )
      translate( xcen, 0.0, 0.0 )
      n  = 0
      while n < strut_info.num do
        -- print_xfrm( )              -- for tests only
        rect( slen, strut_info.width, 1, 0 )
        rotate_z ( strut_info.delta_theta )
        -- print_xfrm( )              -- for tests only
        n = n + 1
      end
    end_subassembly( )
  end

end


--------------------------------------------------------------------------
-- BaffleFuncs.do_baffle
--
-- do a baffle
--
-- Inputs:
--
--   baffle       -- baffle descriptor
--   debug_baffle -- id of baffle to be debugged 
--                   (or 'all' for the whole set)
--
-- where:
--
--  baffle {
--      id             --            identifier
--      z              -- (mm)       axial position of baffle
--      annulus_info   -- table      annulus data
--      strut_info     -- table      strut data
--      displacement   -- table      position    tweaks
--      disorientation -- table      orientation tweaks
--  }

function BaffleFuncs.do_baffle( baffle, debug_baffle )

  begin_subassembly()

    translate( 0, 0, baffle.z)

    if ( baffle.annulus_info.type >= 0 ) then
      -- print_photon( 'raw_project' )
      annulus( baffle.annulus_info.ri, baffle.annulus_info.ro, 0,  
               baffle.annulus_info.type )
      -- print_photon( 'raw_project' )
    end

    if ( baffle.id == debug_baffle or debug_baffle == "all" ) then
      print_photon( 'raw_project' )
    end

    if ( baffle.strut_info.type ~= 'none' ) then
      if ( baffle.strut_info.type == 'strut' ) then
        BaffleFuncs.do_strutpairs( baffle.strut_info )
      elseif ( baffle.strut_info.type == 'rect' ) then
        BaffleFuncs.do_rect_strut( baffle.strut_info )
      end
    end

  end_subassembly( )

end

--------------------------------------------------------------------------
-- BaffleFuncs.deg2rad
--
-- conversion factor for degrees to radians

function BaffleFuncs.deg2rad()

  local  deg2rad = 3.14159265358979/180.0
  return deg2rad

end

--------------------------------------------------------------------------
-- BaffleFuncs.arcmin2rad
--
-- conversion factor for arcmin to radians

function BaffleFuncs.arcmin2rad()

  local  arcmin2rad = 3.14159265358979/(60.0 * 180.0)
  return arcmin2rad

end

--------------------------------------------------------------------------
-- BaffleFuncs.arcsec2rad
--
-- conversion factor for arcsec to radians

function BaffleFuncs.arcsec2rad()

  local  arcsec2rad = 3.14159265358979/(3600.0 * 180.0)
  return arcsec2rad

end

$end
